# -*- coding: utf-8 -*-
"""
Created on Sun Feb 19 16:29:55 2023

@author: synne
""" 
#spot price is in kr/kWh and Energy use is in kWh, while solar irradtiation is Wh/kWp


import pandas as pd
from datetime import datetime
import numpy as np
import matplotlib.pyplot as plt
import math

#imports relevant datafiles
filnavnData = 'importfile/timefortime energiforbruk st.olavs vei.csv'
energiPriser = 'importfile/spotpris bergen 2022.csv'
solarRadiation = 'importfile/solinnstråling 2020 modifisert.csv'

#establishes the amount of solar panels (15 panels, with 550W each)
installed_solar_east = float(3.85) #7 panels
installed_solar_west = float(4.4) #8 panels
installed_solar_optimal = float(4.4) #hypothetical 8 panels

#defines a 5% system loss in the solar panels
system_efficiency = 0.95

#reads files and separates the columns
priser = pd.read_csv(energiPriser, delimiter=",")
df = pd.read_csv(filnavnData, delimiter=";")
solar = pd.read_csv(solarRadiation, delimiter=";")

#renames columns with unpractical names
df = df.rename(columns={"KWH 60 Forbruk": "Energy use"})
df = df.rename(columns={"Fra": "Timestamp"}) 

#makes python able to calculate with the numbers, and adds to the dataframe df
df['Spot'] = priser['value'].copy()
df['Spot'] = df['Spot'].apply(lambda x: float (x))
df['Import Spot'] = df['Spot'].copy() * 1.25

#removes the columns 'Kvalitet' og 'Til'
del df["Kvalitet"]
del df["Til"]

#adds columns in df to see what day of the year and month it is
df["Timestamp"] = df["Timestamp"].apply(lambda x: datetime.strptime(x, "%d.%m.%Y %H:%M"))
df["Month"] = df["Timestamp"].apply(lambda x: x.month)
df["Day of Year"] = df["Timestamp"].apply(lambda x: x.dayofyear)
df["Hour"] = df["Timestamp"].apply(lambda x: x.hour)


#creates columns in the solar dataframe with produced energy depending on installed kWp and system efficiency
solar['Produced west'] = solar['Ppv 1kWp (west 0%)'].apply(lambda x: (x * installed_solar_west)*system_efficiency)
solar['Produced east'] = solar['Ppv 1kWp (east 0%)'].apply(lambda x: (x * installed_solar_east)*system_efficiency)
solar['Produced optimal'] = solar['Ppv 1kWp (optimal 0%)'].apply(lambda x: (x * installed_solar_optimal)*system_efficiency)

#adds the solar production columns to the df dataframe
df['Produced west'] = solar['Produced west'].copy()
df['Produced east'] = solar['Produced east'].copy()
df['Produced south'] = solar['Produced optimal'].copy()

#finds the sum of every hour each month and finds the average energy produced from solar in kWh/day per month
agg = df.groupby(['Month'])['Produced west'].sum()/(df.groupby(['Month'])['Day of Year'].count()/24).apply(lambda x: round(x,0))





#### Energy component BKK grid tariff ####

#defines jan, feb and mar as the first set of months
first_months = [1,2,3]

#defines the cost of the night/weekend/holidays and the day depending if its in the first or second set of months
night_weekend_holidays = [0.3209 if i in first_months else 0.4044 for i in range(1,13)] 
day = [0.4209 if i in first_months else 0.5044 for i in range(1,13)]

#defines what day of the year the holidays in 2022 are out of the 364 days
holidays2022 = [1, 100, 104, 105, 107, 108, 121, 137, 146, 156, 157, 359, 360]

#calculates the cost of energy hour by hour without the capasity component of the grid tariff
#spot is in kr/kWh and Energy use is in kWh
df["Energy component, gridtariff"] = df["Timestamp"].apply(lambda x: night_weekend_holidays[x.month-1] if x.hour < 6 or 22 <= x.hour or x.dayofweek >= 5 or x.dayofyear in holidays2022 else day[x.month-1])
df['Tot price in'] = df['Import Spot'] + df['Energy component, gridtariff']

df['Cost[kr]'] = df['Energy use'] * df['Tot price in']
###########################





#### Calculates the plot of the energy prices ####
#Creates a new df with the needed information
price = df.loc[:, ['Tot price in', "Day of Year", 'Hour']]
price['Price [kr/kWh]'] = df['Tot price in']
del price['Tot price in']

#Creates a table, one column for each day (transposes the values)
table_price = pd.pivot_table(price, values="Price [kr/kWh]", index=["Day of Year"], aggfunc=np.mean, columns=["Hour"], fill_value=0).T
table_price_two = table_price.copy()

#Calculates the mean value. Creates a list of 24 values (attached to the time of day).
table_price_two = table_price_two.mean(axis = 1)
table_price_two = table_price_two.sort_values(ascending = True)

#Sorts the table so each column is in acending order
for col in table_price:
    table_price[col] = table_price[col].sort_values(ignore_index=True)

#Calculates the mean value of every lowest value, all the second lowest value etc. Creates a list of 24 values (not attached to the time of day).
mean_hours = table_price.mean(axis = 1)

#Calculates one value for every day, max value - min value, gives the delta of the energy price each day
delta_day = table_price.T.apply(lambda x: x[23] - x[0], axis = 1)

#Puts the delta_day in acending order, and changes the column names to 'day' and 'Value'
delta_day_ascending = delta_day.sort_values(ascending = True)
delta_day_ascending = pd.DataFrame({'day': delta_day_ascending.index, 'Value': delta_day_ascending.values})

###plots the table_price_two
table_price_two = table_price_two.reset_index()

#Sets the labels  and ticks etc. for the table_price_two plot
xpoints = np.array([i for i in range(24)])
xpoints_label = table_price_two['Hour'].values


ax = plt.gca()
plt.title("Energyprice mean every hour, Bergen", loc = 'left')
plt.xlabel("Hour")
plt.ylabel("Energyprice [NOK/kWh]")
table_price_two[0].plot(kind='line',marker='o', x='Hour' ,y=1, color='green', ax=ax)
ax.set_xticks(xpoints)
ax.set_xticklabels(xpoints_label)

###plots the mean_hours plot
plt.show()

#Sets the labels and ticks etc. for the mean_hours plot
xpoints = np.array([i for i in range(24)])
xpoints_label = np.array([str(i) for i in range(24)])

ax = plt.gca()
plt.title("Energyprice mean of cheapest to most expensive hour, Bergen", loc = 'left')
plt.xlabel("Hour")
plt.ylabel("Energyprice [NOK/kWh]")
mean_hours.plot(kind='line',marker='o', x=xpoints ,y=1, color='green', ax=ax)
ax.set_xticks(xpoints)
ax.set_xticklabels(xpoints_label)

#plots the mean_hours plot
plt.show()

#Sets the labels and ticks etc. for the delta_day_acending plot
xpoints = np.array([0, 50, 100, 150, 200, 250, 300, 365])
xpoints_label = np.array(['0', '50', '100', '150', '200', '250', '300', '365'])

ax = plt.gca()
plt.title("Delta max-min value Acending", loc = 'left')
plt.xlabel("Days")
plt.ylabel("Energyprice [NOK/kWh]")
delta_day_ascending['Value'].plot(kind='line', color='green', ax=ax)
ax.set_xticks(xpoints)
ax.set_xticklabels(xpoints_label)

#plots the delta_day_acending plot
plt.show()
###########################





#### Capasity component BKK grid tariff ####

#Finds the peak hour of every day
df_aggregated = (df.groupby(["Month","Day of Year"])["Energy use"].max()) 
df_aggregated = df_aggregated.reset_index()

peaks = []
tierPeaks = []
additional_cost = []

#Finds the mean of the three highest values and the sum of the energy use per month
for i in range(1,13):  
    daily_peaks = df[df["Month"] == i]
    daily_peaks = daily_peaks.sort_values(by='Energy use',ascending=False)
    peaks.append(daily_peaks["Energy use"][:3].mean())
    

#Checks what tier mean of the three highest values per month corresponds to and the associated additional cost
for value in peaks:
    if 0 <= value and value < 2:
        tierPeaks.append('Tier 1')
        additional_cost.append(125)
    elif 2 <= value and value < 5:
        tierPeaks.append('Tier 2')
        additional_cost.append(206)
    elif 5 <= value and value < 10:
        tierPeaks.append('Tier 3')
        additional_cost.append(350)
    elif 10 <= value and value < 15:
        tierPeaks.append('Tier 4')
        additional_cost.append(494)
    elif 15 <= value and value < 20:
        tierPeaks.append('Tier 5')
        additional_cost.append(638)
    elif 20 <= value and value < 25:
        tierPeaks.append('Tier 6')
        additional_cost.append(781)

##################################################





########summarizes data#########################
   
month_sum_energy = []
month_sum_cost = []
month_sum_cost_noaddition = []

year_energycost = 0
energy_year_sum = 0

month_energyproduction_west = []
month_energyproduction_east = []
month_energyproduction_total = []

energyproduction_sum_year_west = 0
energyproduction_sum_year_east = 0
energyproduction_sum_year_total = 0 

#Finds the total cost each month by adding the cost hour by hour (energy use * (spot + energy component)) from the dataframe df, and adding the additional cost per month
for i in range(1,13):
    month_df = df[df["Month"] == i]
    month_sum_cost.append((month_df["Cost[kr]"].sum()) + additional_cost[i-1])
    month_sum_cost_noaddition.append(month_df["Cost[kr]"].sum())
    month_sum_energy.append(month_df["Energy use"].sum())
    
    month_energyproduction_west.append(month_df['Produced west'].sum())
    month_energyproduction_east.append(month_df['Produced east'].sum())
    month_energyproduction_total.append(month_df['Produced west'].sum() + month_df['Produced east'].sum())

for x in month_sum_cost:
    year_energycost += x

for x in month_sum_energy:
    energy_year_sum += x
    
for x in month_energyproduction_west:
    energyproduction_sum_year_west += x
    energyproduction_sum_year_total += x
    
for x in month_energyproduction_east:
    energyproduction_sum_year_east += x
    energyproduction_sum_year_total += x

###################################
# print("The energycost in St.Olavs vei 170 in 2022 was " + str(math.floor(year_energycost)) + " NOK.")
# print("The average monthly energycost in St.Olavs vei 170 in 2022 was " + str(math.floor(year_energycost/12)) + " NOK.")
# print("The total energy use in St.Olavs vei 170 in 2022 was " + str(math.floor(energy_year_sum)) + " kWh.")


#####
# print(energyproduction_sum_year_west)
# print(month_energyproduction_west)

# print(energyproduction_sum_year_east)
# print(month_energyproduction_east)

# print(energyproduction_sum_year_total)
# print(month_energyproduction_total)

####de over er i Wh ikke kWh

# print(energyproduction_sum_year_total/1000)

# print(month_sum_cost)
# print(month_sum_cost_noaddition)
# print(month_sum_energy)

# print(year_energycost)
# # print(energy_year_sum)
  

# print(tierPeaks)
# print(additional_cost)
# print(peaks)
# ##########################

# df.to_csv('StOlavsResultDfFull.csv')
# df.rename(columns = {'Energy use':'Other loads [kWh]'}, inplace = True)
# df['Heating tank [kWh]'] = 0
# df['Electric car [kWh]'] = 0

# result =  df.copy().drop(columns={'Import Spot', 'Hour', 'Energy component, gridtariff', 'Cost[kr]'})
# result.to_csv('StOlavsveiResultDf.csv')